-----------------------------------------------------------------------------
--
--	UART_RX.VHDL
--
--
--	Author 	: Rene Bllhoff
--  Created : Feb. 2007
--
--	(c) 2005-2007 by Rene Bllhoff - reneboellhoff (at) gmx (dot) de - 
-----------------------------------------------------------------------------
--
--  This source file may be used and distributed without
--  restriction provided that this copyright statement is not
--  removed from the file and that any derivative work contains
--  the original copyright notice and the associated disclaimer.
--
--  This program is free software; you can redistribute it and/or
--  modify it under the terms of the GNU General Public License
--  as published by the Free Software Foundation; either version 2
--  of the License, or (at your option) any later version.
--										
--  This program is distributed in the hope that it will be useful,
--  but WITHOUT ANY WARRANTY; without even the implied warranty of
--  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.See the GNU
--  General Public License for more details.
--
--  You should have received a copy of the GNU General Public
--  License along with this source; if not, download it	from
--  http://www.gnu.org/licenses/gpl.html	
--
-----------------------------------------------------------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity uart_rx is 
  port (clk 				: in 	std_logic;
        res 				: in 	std_logic;
        rx_in 			: in 	std_logic;
        baud_clk_16 : in 	std_logic;
        busy 				: out std_logic;
        out_buffer 	: out std_logic_vector (7 downto 0);
        trg 				: out std_logic);
end uart_rx;


architecture behave of uart_rx is
  
  signal state 					: std_logic_vector (3 downto 0);
  signal count 					: std_logic_vector (3 downto 0);
  signal baud_clk_16_z1 : std_logic;
  signal rx_buffer 			: std_logic_vector (7 downto 0);
  signal err 						: std_logic;

  signal rx_in_z1				: std_logic;
  signal rx_in_z2				: std_logic;
  signal rx_in_z3				: std_logic;

  signal trg_bc16				: std_logic;
  signal trg_bc16_z1		: std_logic;
  

begin


  process (clk,res)
  begin
    if res = '1' then
      state 					<= (others => '0');
      count 					<= (others => '0');
      rx_buffer 			<= (others => '0');
      baud_clk_16_z1 	<= '0';
      err 						<= '0';
			trg_bc16 				<= '0';
			trg_bc16_z1			<= '0';
  
    elsif clk'event and clk = '1' then
      baud_clk_16_z1 <= baud_clk_16;


      if baud_clk_16_z1 = '0' and baud_clk_16 = '1' then

				rx_in_z1 <= rx_in   ;
				rx_in_z2 <= rx_in_z1;
				rx_in_z3 <= rx_in_z2;

				count <= count + '1';

        if state = X"0" then
			    if rx_in_z3 = '1' and rx_in_z2 = '0' and rx_in_z1 = '0' then
					  count <= (others => '0');
						state <= X"1"; 
			    end if;
					trg_bc16 				<= '0';	 -- trigger resetten 
  			else
			    if count = X"8" then
						case state is
              when X"1" => if rx_in_z3 = '0' then state <= X"2"; else state <= X"0"; end if;
              when X"2" => rx_buffer (0) <= rx_in_z3; state <= X"3";
              when X"3" => rx_buffer (1) <= rx_in_z3; state <= X"4";
              when X"4" => rx_buffer (2) <= rx_in_z3; state <= X"5";
              when X"5" => rx_buffer (3) <= rx_in_z3; state <= X"6";
              when X"6" => rx_buffer (4) <= rx_in_z3; state <= X"7";
              when X"7" => rx_buffer (5) <= rx_in_z3; state <= X"8";
              when X"8" => rx_buffer (6) <= rx_in_z3; state <= X"9";
              when X"9" => rx_buffer (7) <= rx_in_z3; state <= X"a";
              when X"a" => if rx_in_z3 = '1' then
                             out_buffer <= rx_buffer;
                             err <= '0';
                             trg_bc16 				<= '1';
                           else
														 out_buffer <= (others => '0');
                             err <= '1';
                           end if;
													 state <= X"0";
              when others => NULL;
            end case;

          end if;

        end if;      

      end if;

			trg_bc16_z1	<= trg_bc16;
			trg 				<= trg_bc16 and not trg_bc16_z1;

    end if;
  end process;

end architecture;